@@ -5,13 +5,14 @@ The final `Indexer`_ classes exposes a high-level Searcher and Writer.
 """
 
 from future_builtins import map, zip
+import warnings
 import os
 import re
 import itertools, operator
 import contextlib
 import abc, collections
 import lucene
-from .queries import Query, HitCollector, Filter, SortField, Highlighter, SpellChecker, SpellParser
+from .queries import Query, Collector, SortField, Highlighter, FastVectorHighlighter, SpellChecker, SpellParser
 from .documents import Field, Document, Hits
 
 class Atomic(object):
@@ -28,27 +29,44 @@ class closing(set):
     def __del__(self):
         for obj in self:
             obj.close()
+    def analyzer(self, analyzer, version=None):
+        if analyzer is None:
+            analyzer = lucene.StandardAnalyzer(version or lucene.Version.values()[-1])
+            self.add(analyzer)
+        return analyzer
+    def directory(self, directory):
+        if directory is None:
+            directory = lucene.RAMDirectory()
+            self.add(directory)
+        elif isinstance(directory, basestring):
+            directory = lucene.FSDirectory.open(lucene.File(directory))
+            self.add(directory)
+        return directory
+    def reader(self, reader):
+        reader = self.directory(reader)
+        if isinstance(reader, lucene.Searcher):
+            warnings.warn('Support for opening searchers is deprecated; call with a reader instead.', DeprecationWarning)
+            reader = reader.indexReader
+        if isinstance(reader, lucene.IndexReader):
+            reader.incRef()
+        else:
+            reader = lucene.IndexReader.open(reader)
+        return reader
 
 class TokenFilter(lucene.PythonTokenFilter):
     """Create an iterable lucene TokenFilter from a TokenStream.
-    In lucene version 2, call on any iterable of tokens.
-    In lucene version 3, subclass and override :meth:`incrementToken`;
-    attributes are cached as properties to create a Token interface.
+    Subclass and override :meth:`incrementToken`.
+    Attributes are cached as properties to create a Token interface.
     """
     def __init__(self, input):
-        if issubclass(lucene.TokenFilter, collections.Iterable):
-            lucene.PythonTokenFilter.__init__(self, lucene.EmptyTokenStream())
-            self.next = iter(input).next
-        else:
-            lucene.PythonTokenFilter.__init__(self, input)
-            self.input = input
+        lucene.PythonTokenFilter.__init__(self, input)
+        self.input = input
     def __iter__(self):
         return self
-    if not hasattr(lucene.TokenFilter, 'next'):
-        def next(self):
-            if self.incrementToken():
-                return self
-            raise StopIteration
+    def next(self):
+        if self.incrementToken():
+            return self
+        raise StopIteration
     def incrementToken(self):
         "Advance to next token and return whether the stream is not empty."
         return self.input.incrementToken()
@@ -107,23 +125,22 @@ class Analyzer(lucene.PythonAnalyzer):
         tokens = self.tokenizer.tokenStream(field, reader) if isinstance(self.tokenizer, lucene.Analyzer) else self.tokenizer(reader)
         for filter in self.filters:
             tokens = filter(tokens)
-        return tokens if isinstance(tokens, lucene.TokenStream) else TokenFilter(tokens)
+        return tokens
     def tokens(self, text, field=None):
         "Return lucene TokenStream from text."
         return self.tokenStream(field, lucene.StringReader(text))
-    def parse(self, query, field='', op='', version=lucene.VERSION[:3], parser=None, **attrs):
+    def parse(self, query, field='', op='', version='', parser=None, **attrs):
         """Return parsed lucene Query.
         
         :param query: query string
         :param field: default query field name, sequence of names, or boost mapping
         :param op: default query operator ('or', 'and')
-        :param version: lucene Version string, leave blank for deprecated constructor
+        :param version: lucene Version string
         :param parser: custom PythonQueryParser class
         :param attrs: additional attributes to set on the parser
         """
         # parsers aren't thread-safe (nor slow), so create one each time
-        version = getattr(lucene.Version, 'LUCENE_' + version.replace('.', ''))
-        args = [] if lucene.VERSION.startswith('2.9') and parser is not None else [version]
+        args = [lucene.Version.valueOf('LUCENE_' + version.replace('.', '')) if version else lucene.Version.values()[-1]]
         if isinstance(field, collections.Mapping):
             boosts = lucene.HashMap()
             for key in field:
@@ -137,7 +154,7 @@ class Analyzer(lucene.PythonAnalyzer):
         for name, value in attrs.items():
             setattr(parser, name, value)
         if isinstance(parser, lucene.MultiFieldQueryParser):
-            return lucene.MultiFieldQueryParser.parse(parser, query) # bug in method binding
+            return lucene.MultiFieldQueryParser.parse(parser, query)
         try:
             return parser.parse(query)
         finally:
@@ -171,6 +188,10 @@ class IndexReader(object):
     def timestamp(self):
         "timestamp of reader's last commit"
         return self.indexCommit.timestamp / 1000.0
+    @property
+    def segments(self):
+        "segment filenames with document counts"
+        return dict((lucene.SegmentReader.cast_(reader).segmentName, reader.numDocs()) for reader in self.sequentialSubReaders)
     def copy(self, dest, query=None, exclude=None, optimize=False):
         """Copy the index to the destination directory.
         Optimized to use hard links if the destination is a file system path.
@@ -228,13 +249,11 @@ class IndexReader(object):
         elif '*' in value or '?' in value:
             termenum = lucene.WildcardTermEnum(self.indexReader, term)
         else:
-            termenum = self.indexReader.terms(term)
+            termenum = lucene.TermRangeTermEnum(self.indexReader, name, value, stop, True, False, None)
         with contextlib.closing(termenum):
             term = termenum.term()
-            while term and term.field() == name:
+            while term:
                 text = term.text()
-                if stop is not None and text >= stop:
-                    break
                 yield (text, termenum.docFreq()) if counts else text
                 term = termenum.next() and termenum.term()
     def numbers(self, name, step=0, type=int, counts=False):
@@ -333,37 +352,38 @@ class IndexReader(object):
                     raise
         return int(count)
 
-class Searcher(object):
-    "Mixin interface common among searchers."
-    def __init__(self, arg, analyzer=None):
-        super(Searcher, self).__init__(arg)
-        if analyzer is None:
-            analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT)
-            self.shared.add(analyzer)
-        self.analyzer = analyzer
+class IndexSearcher(lucene.IndexSearcher, IndexReader):
+    """Inherited lucene IndexSearcher, with a mixed-in IndexReader.
+    
+    :param directory: directory path, lucene Directory, or lucene IndexReader
+    :param analyzer: lucene Analyzer, default StandardAnalyzer
+    """
+    def __init__(self, directory, analyzer=None):
+        self.shared = closing()
+        lucene.IndexSearcher.__init__(self, self.shared.directory(directory))
+        self.analyzer = self.shared.analyzer(analyzer)
         self.filters, self.sorters, self.spellcheckers = {}, {}, {}
+    @classmethod
+    def load(cls, directory, analyzer=None):
+        "Open `IndexSearcher`_ with a lucene RAMDirectory, loading index into memory."
+        ref = closing()
+        self = cls(lucene.RAMDirectory(ref.directory(directory)), analyzer)
+        self.shared.add(self.directory)
+        return self
     def __del__(self):
         if hash(self):
             self.close()
-    def open(self, *directories):
-        self.shared = closing()
-        for directory in directories:
-            if isinstance(directory, basestring):
-                directory = lucene.FSDirectory.open(lucene.File(directory))
-                self.shared.add(directory)
-            yield directory
     def reopen(self, filters=False, sorters=False, spellcheckers=False):
-        """Return current `Searcher`_, only creating a new one if necessary.
+        """Return current `IndexSearcher`_, only creating a new one if necessary.
         
         :param filters: refresh cached facet :attr:`filters`
         :param sorters: refresh cached :attr:`sorters` with associated parsers
         :param spellcheckers: refresh cached :attr:`spellcheckers`
         """
-        reader = self.indexReader.reopen()
-        if reader == self.indexReader:
+        if self.current:
             return self
-        other = type(self)(reader, self.analyzer)
-        other.shared, other.owned = self.shared, closing([reader])
+        other = type(self)(self.indexReader.reopen(), self.analyzer)
+        other.shared, other.owned = self.shared, closing([other.indexReader])
         other.filters.update((key, value if isinstance(value, lucene.Filter) else dict(value)) for key, value in self.filters.items())
         if filters:
             other.facets(Query.any(), *other.filters)
@@ -388,38 +408,30 @@ class Searcher(object):
         if spellcheck:
             kwargs['parser'], kwargs['searcher'] = SpellParser, self
         return Analyzer.__dict__['parse'](self.analyzer, query, **kwargs)
-    def highlighter(self, query, span=True, formatter=None, encoder=None, field=None):
-        "Return `Highlighter`_ specific to the searcher's analyzer and index."
-        query = self.parse(query, field=field or '')
-        return Highlighter(query, self.analyzer, span, formatter, encoder, field, (field and self.indexReader))
-    def highlight(self, query, doc, count=1, span=True, formatter=None, encoder=None, field=None, **attrs):
-        """Return highlighted text fragments which match the query, using internal :meth:`highlighter`.
-        
-        :param query: query string or lucene Query
-        :param doc: text string or doc id to be highlighted
-        :param count: maximum number of fragments
-        :param attrs: additional attributes to set on the highlighter
-        """
-        highlighter = self.highlighter(query, span, formatter, encoder, field)
-        for name, value in attrs.items():
-            setattr(highlighter, name, value)
-        if not isinstance(doc, basestring):
-            doc = self[doc][field]
-        return highlighter.fragments(doc, count)
+    def highlighter(self, query, field, **kwargs):
+        "Return `Highlighter`_ or if applicable `FastVectorHighlighter`_ specific to searcher and query."
+        query = self.parse(query, field=field)
+        vector = hasattr(lucene, 'FastVectorHighlighter') and field in self.names('termvector_with_position_offset')
+        return (FastVectorHighlighter if vector else Highlighter)(self, query, field, **kwargs)
     def count(self, *query, **options):
         """Return number of hits for given query or term.
         
         :param query: :meth:`search` compatible query, or optimally a name and value
         :param options: additional :meth:`search` options
         """
-        if len(query) <= 1:
-            return self.search(*query, count=1, sort=lucene.Sort.INDEXORDER, **options).count
-        return self.docFreq(lucene.Term(*query))
+        if len(query) > 1:
+            return self.docFreq(lucene.Term(*query))
+        query = self.parse(*query, **options) if query else lucene.MatchAllDocsQuery()
+        if not hasattr(lucene, 'TotalHitCountCollector'):
+            return lucene.IndexSearcher.search(self, query, options.get('filter'), 1, lucene.Sort.INDEXORDER).totalHits
+        collector = lucene.TotalHitCountCollector()
+        lucene.IndexSearcher.search(self, query, options.get('filter'), collector)
+        return collector.totalHits
     def search(self, query=None, filter=None, count=None, sort=None, reverse=False, scores=False, maxscore=False, timeout=None, **parser):
         """Run query and return `Hits`_.
         
         :param query: query string or lucene Query
-        :param filter: doc ids or lucene Filter
+        :param filter: lucene Filter
         :param count: maximum number of hits to retrieve
         :param sort: if count is given, lucene Sort parameters, else a callable key
         :param reverse: reverse flag used with sort
@@ -429,12 +441,10 @@ class Searcher(object):
         :param parser: :meth:`Analyzer.parse` options
         """
         query = lucene.MatchAllDocsQuery() if query is None else self.parse(query, **parser)
-        if not isinstance(filter, (lucene.Filter, type(None))):
-            filter = Filter(filter)
         weight = query.weight(self)
-        # use custom HitCollector if all results are necessary, otherwise use lucene's TopDocsCollectors
+        # use custom Collector if all results are necessary, otherwise use lucene's TopDocsCollectors
         if count is None:
-            collector = HitCollector()
+            collector = Collector()
         else:
             count, inorder = min(count, self.maxDoc()), not weight.scoresDocsOutOfOrder()
             if sort is None:
@@ -445,14 +455,13 @@ class Searcher(object):
                 if not isinstance(sort, lucene.Sort):
                     sort = lucene.Sort(sort)
                 collector = lucene.TopFieldCollector.create(sort, count, False, scores, maxscore, inorder)
-        TimeLimitingCollector = lucene.TimeLimitingCollector if isinstance(collector, lucene.Collector) else lucene.TimeLimitedCollector
-        results = collector if timeout is None else TimeLimitingCollector(collector, long(timeout * 1000))
+        results = collector if timeout is None else lucene.TimeLimitingCollector(collector, long(timeout * 1000))
         try:
-            super(Searcher, self).search(weight, filter, results)
+            lucene.IndexSearcher.search(self, weight, filter, results)
         except lucene.JavaError as timeout:
-            if not TimeLimitingCollector.TimeExceededException.instance_(timeout.getJavaException()):
+            if not lucene.TimeLimitingCollector.TimeExceededException.instance_(timeout.getJavaException()):
                 raise
-        if isinstance(collector, HitCollector):
+        if isinstance(collector, Collector):
             ids, scores = collector.sorted(key=sort, reverse=reverse)
             collector.finalize()
             stats = len(ids), max(scores or [float('nan')])
@@ -474,9 +483,7 @@ class Searcher(object):
             query = self.parse(query)
         if isinstance(query, lucene.Query):
             query = lucene.QueryWrapperFilter(query)
-        if not isinstance(query, lucene.Filter):
-            query = Filter(query)
-        elif not isinstance(query, lucene.CachingWrapperFilter):
+        if not isinstance(query, lucene.CachingWrapperFilter):
             query = lucene.CachingWrapperFilter(query)
         for key in keys:
             filters = self.filters.get(key)
@@ -539,51 +546,33 @@ class Searcher(object):
         for query in queries:
             yield searcher.search(self.parse(query))
 
-class IndexSearcher(Searcher, lucene.IndexSearcher, IndexReader):
-    """Inherited lucene IndexSearcher, with a mixed-in IndexReader.
+class MultiSearcher(IndexSearcher):
+    """IndexSearcher with underlying lucene MultiReader.
     
-    :param directory: directory path, lucene Directory, or lucene IndexReader
+    :param reader: directory paths, Directories, IndexReaders, or a single MultiReader
     :param analyzer: lucene Analyzer, default StandardAnalyzer
     """
-    def __init__(self, directory, analyzer=None):
-        Searcher.__init__(self, *self.open(directory), analyzer=analyzer)
-    @classmethod
-    def load(cls, directory, analyzer=None):
-        "Open `Searcher`_ with a lucene RAMDirectory, loading index into memory."
-        if isinstance(directory, basestring):
-            directory = lucene.FSDirectory.open(lucene.File(directory))
-            ref = closing([directory])
-        self = cls(lucene.RAMDirectory(directory), analyzer)
-        self.shared.add(self.directory)
-        return self
-
-class MultiSearcher(Searcher, lucene.MultiSearcher, IndexReader):
-    """Inherited lucene MultiSearcher.
-    All sub searchers will be closed when the MultiSearcher is closed.
-    
-    :param searchers: directory paths, Directories, IndexReaders, Searchers, or a MultiReader
-    :param analyzer: lucene Analyzer, default StandardAnalyzer
-    """
-    def __init__(self, searchers, analyzer=None):
-        if lucene.MultiReader.instance_(searchers):
-            self.indexReader = searchers
-            searchers = list(map(lucene.IndexSearcher, searchers.sequentialSubReaders))
-        else:
-            searchers = [searcher if isinstance(searcher, lucene.Searcher) else lucene.IndexSearcher(searcher) for searcher in self.open(*searchers)]
-            self.indexReader = lucene.MultiReader(list(map(operator.attrgetter('indexReader'), searchers)), False)
-            self.owned = closing([self.indexReader])
-        Searcher.__init__(self, searchers, analyzer)
+    def __init__(self, reader, analyzer=None):
+        shared = closing()
+        if not lucene.MultiReader.instance_(reader):
+            reader = lucene.MultiReader(list(map(shared.reader, reader)), True)
+            self.owned = closing([reader])
+        IndexSearcher.__init__(self, reader, analyzer)
+        self.shared.update(shared)
+        shared.clear()
     @property
     def version(self):
-        return ' '.join(str(reader.version) for reader in self.sequentialSubReaders)
+        return sum(map(operator.attrgetter('version'), self.sequentialSubReaders))
     @property
     def timestamp(self):
         return max(IndexReader(reader).timestamp for reader in self.sequentialSubReaders)
     def overlap(self, *filters):
         return sum(IndexReader(reader).overlap(*filters) for reader in self.sequentialSubReaders)
 
-class ParallelMultiSearcher(MultiSearcher, lucene.ParallelMultiSearcher):
-    "Inherited lucene ParallelMultiSearcher."
+class ParallelMultiSearcher(MultiSearcher):
+    def __init__(self, *args, **kwargs):
+        warnings.warn('ParallelMultiSearchers have been deprecated; use MultiSearcher.', DeprecationWarning)
+        MultiSearcher.__init__(self, *args, **kwargs)
 
 class IndexWriter(lucene.IndexWriter):
     """Inherited lucene IndexWriter.
@@ -592,26 +581,30 @@ class IndexWriter(lucene.IndexWriter):
     :param directory: directory path or lucene Directory, default RAMDirectory
     :param mode: file mode (rwa), except updating (+) is implied
     :param analyzer: lucene Analyzer, default StandardAnalyzer
+    :param version: lucene Version argument passed to IndexWriterConfig or StandardAnalyzer, default is latest
     """
     __len__ = lucene.IndexWriter.numDocs
-    __del__ = Searcher.__dict__['__del__']
-    parse = Searcher.__dict__['parse']
-    def __init__(self, directory=None, mode='a', analyzer=None):
+    __del__ = IndexSearcher.__dict__['__del__']
+    parse = IndexSearcher.__dict__['parse']
+    def __init__(self, directory=None, mode='a', analyzer=None, version=None):
         self.shared = closing()
-        if analyzer is None:
-            analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT)
-            self.shared.add(analyzer)
-        if not isinstance(directory, lucene.Directory):
-            directory = lucene.RAMDirectory() if directory is None else lucene.FSDirectory.open(lucene.File(directory))
-            self.shared.add(directory)
-        args = [] if mode == 'a' else [bool('rw'.index(mode))]
-        args.append(lucene.IndexWriter.MaxFieldLength.LIMITED)
-        lucene.IndexWriter.__init__(self, directory, analyzer, *args)
+        if version is None:
+            version = lucene.Version.values()[-1]
+        analyzer = self.shared.analyzer(analyzer, version)
+        directory = self.shared.directory(directory)
+        if hasattr(lucene, 'IndexWriterConfig'):
+            config = lucene.IndexWriterConfig(version, analyzer)
+            config.openMode = lucene.IndexWriterConfig.OpenMode.values()['wra'.index(mode)]
+            lucene.IndexWriter.__init__(self, directory, config)
+        else:
+            args = [] if mode == 'a' else [bool('rw'.index(mode))]
+            args.append(lucene.IndexWriter.MaxFieldLength.UNLIMITED)
+            lucene.IndexWriter.__init__(self, directory, analyzer, *args)
         self.fields = {}
     @property
     def segments(self):
-        "segment filenames with document counts"
-        return dict((name, int(value)) for name, value in re.findall('(\w+).*:.x?(\d+)', self.segString()))
+        warnings.warn('IndexWriter.segString is for Lucene internal purposes only', DeprecationWarning)
+        return dict((name, int(value)) for name, value in re.findall('(\w+).*?:.x?(\d+)', self.segString()))
     def set(self, name, cls=Field, **params):
         """Assign parameters to field name.
         
@@ -638,11 +631,11 @@ class IndexWriter(lucene.IndexWriter):
         "Atomically delete documents which match given term and add the new :meth:`document` with optional boost."
         doc = self.document(document, **terms)
         doc.boost = boost
-        self.updateDocument(lucene.Term(name, *([value] if value else doc.getValues(name))), doc)
+        self.updateDocument(lucene.Term(name, *[value] if value else doc.getValues(name)), doc)
     def delete(self, *query, **options):
         """Remove documents which match given query or term.
         
-        :param query: :meth:`Searcher.search` compatible query, or optimally a name and value
+        :param query: :meth:`IndexSearcher.search` compatible query, or optimally a name and value
         :param options: additional :meth:`Analyzer.parse` options
         """
         parse = self.parse if len(query) == 1 else lucene.Term
@@ -659,11 +652,22 @@ class IndexWriter(lucene.IndexWriter):
 class Indexer(IndexWriter):
     """An all-purpose interface to an index.
     Creates an `IndexWriter`_ with a delegated `IndexSearcher`_.
+    
+    :param nrt: optionally use a near real-time searcher
     """
-    def __init__(self, *args, **kwargs):
-        IndexWriter.__init__(self, *args, **kwargs)
+    def __init__(self, directory=None, mode='a', analyzer=None, version=None, nrt=False):
+        IndexWriter.__init__(self, directory, mode, analyzer, version)
         IndexWriter.commit(self)
-        self.indexSearcher = IndexSearcher(self.directory, self.analyzer)
+        self.nrt = nrt
+        if nrt:
+            try:
+                reader = lucene.IndexReader.open(self, True)
+            except lucene.InvalidArgsError:
+                reader = self.reader
+            self.indexSearcher = IndexSearcher(reader, self.analyzer)
+            self.indexSearcher.owned = closing([reader])
+        else:
+            self.indexSearcher = IndexSearcher(self.directory, self.analyzer)
     def __getattr__(self, name):
         if name == 'indexSearcher':
             raise AttributeError(name)
@@ -674,12 +678,17 @@ class Indexer(IndexWriter):
         return iter(self.indexSearcher)
     def __getitem__(self, id):
         return self.indexSearcher[id]
+    @property
+    def segments(self):
+        return self.indexSearcher.segments
+    def refresh(self, **caches):
+        "Store refreshed searcher with :meth:`IndexSearcher.reopen` caches."
+        self.indexSearcher = self.indexSearcher.reopen(**caches)
     def commit(self, expunge=False, optimize=False, **caches):
-        """Commit writes and refresh searcher.  Not thread-safe.
+        """Commit writes and :meth:`refresh` searcher.
         
         :param expunge: expunge deletes
         :param optimize: optimize index, optionally supply number of segments
-        :param caches: :meth:`Searcher.reopen` caches to refresh
         """
         IndexWriter.commit(self)
         if expunge:
@@ -688,4 +697,4 @@ class Indexer(IndexWriter):
         if optimize:
             self.optimize(optimize)
             IndexWriter.commit(self)
-        self.indexSearcher = self.indexSearcher.reopen(**caches)
+        self.refresh(**caches)
