@@ -3,7 +3,6 @@ Query wrappers and search utilities.
 """
 
 from future_builtins import filter, map
-import warnings
 import itertools
 import bisect
 import heapq
@@ -23,9 +22,9 @@ class Query(object):
         return lucene.CachingWrapperFilter(filter) if cache else filter
     def terms(self):
         "Generate set of query term items."
-        terms = lucene.HashSet()
+        terms = lucene.HashSet().of_(lucene.Term)
         self.extractTerms(terms)
-        for term in map(lucene.Term.cast_, terms):
+        for term in terms:
             yield term.field(), term.text()
     @classmethod
     def term(cls, name, value):
@@ -140,8 +139,11 @@ class BooleanQuery(Query):
 class SpanQuery(Query):
     "Inherited lucene SpanQuery with additional span constructors."
     def __getitem__(self, slc):
-        assert slc.start is slc.step is None, 'only prefix slice supported'
-        return SpanQuery(lucene.SpanFirstQuery, self, slc.stop)
+        start, stop, step = slc.indices(lucene.Integer.MAX_VALUE)
+        assert step == 1, 'slice step is not supported'
+        if start == 0:
+            return SpanQuery(lucene.SpanFirstQuery, self, stop)
+        return SpanQuery(lucene.SpanPositionRangeQuery, self, start, stop)
     def __sub__(self, other):
         return SpanQuery(lucene.SpanNotQuery, self, other)
     def __or__(*spans):
@@ -158,12 +160,18 @@ class SpanQuery(Query):
     def mask(self, name):
         "Return lucene FieldMaskingSpanQuery, which allows combining span queries from different fields."
         return SpanQuery(lucene.FieldMaskingSpanQuery, self, name)
+    def payload(self, *values):
+        "Return lucene SpanPayloadCheckQuery from payload values."
+        matches = lucene.ArrayList()
+        for value in values:
+            matches.add(lucene.JArray_byte(value))
+        base = lucene.SpanNearPayloadCheckQuery if lucene.SpanNearQuery.instance_(self) else lucene.SpanPayloadCheckQuery
+        return SpanQuery(base, self, matches)
 
-class HitCollector(lucene.PythonCollector if hasattr(lucene, 'PythonCollector') else lucene.PythonHitCollector):
+class Collector(lucene.PythonCollector):
     "Collect all ids and scores efficiently."
     def __init__(self):
-        super(HitCollector, self).__init__()
-        self.base = 0
+        lucene.PythonCollector.__init__(self)
         self.scores = {}
     def collect(self, id, score):
         self.scores[id + self.base] = score
@@ -179,34 +187,6 @@ class HitCollector(lucene.PythonCollector if hasattr(lucene, 'PythonCollector')
         ids.sort(key=key, reverse=reverse)
         return ids, list(map(self.scores.__getitem__, ids))
 
-class Filter(lucene.PythonFilter):
-    "Inherited lucene Filter with a cached BitSet of ids."
-    def __init__(self, ids):
-        warnings.warn('Filters have been deprecated and are incompatible with segment based searching.', DeprecationWarning)
-        lucene.PythonFilter.__init__(self)
-        self.docIdSet = lucene.OpenBitSet()
-        if isinstance(ids, lucene.OpenBitSet):
-            self.docIdSet.union(ids)
-        else:
-            setter = self.docIdSet.set
-            for id in map(long, ids):
-                setter(id)
-        if lucene.VERSION < '3':
-            self.bitSet = lucene.BitSet()
-            setter = self.bitSet.set
-            for id in filter(self.docIdSet.get, xrange(self.docIdSet.size())):
-                setter(id)
-    @staticmethod
-    def overlap(self, other, reader=None):
-        "Return intersection count of the filters."
-        return int(lucene.OpenBitSet.intersectionCount(self.getDocIdSet(reader), other.getDocIdSet(reader)))
-    def bits(self, reader=None):
-        "Return cached BitSet, reader is ignored.  Deprecated."
-        return self.bitSet
-    def getDocIdSet(self, reader=None):
-        "Return cached OpenBitSet, reader is ignored."
-        return self.docIdSet
-
 class SortField(lucene.SortField):
     """Inherited lucene SortField used for caching FieldCache parsers.
         
@@ -227,33 +207,65 @@ class SortField(lucene.SortField):
     def comparator(self, reader):
         "Return indexed values from default FieldCache using the given reader."
         method = getattr(lucene.FieldCache.DEFAULT, 'get{0}s'.format(self.typename))
-        return method(reader, self.field, *([self.parser] * bool(self.parser)))
+        return method(reader, self.field, *[self.parser] * bool(self.parser))
 
 class Highlighter(lucene.Highlighter):
-    """Inherited lucene Filter with stored analysis options.
+    """Inherited lucene Highlighter with stored analysis options.
     
+    :param searcher: `IndexSearcher`_ used for analysis, scoring, and optionally text retrieval
     :param query: lucene Query
-    :param analyzer: analyzer for texts
-    :param span: only highlight terms which would contribute to a hit
-    :param formatter: optional lucene Formatter or html tag name
+    :param field: field name of text
+    :param terms: highlight any matching term in query regardless of position
+    :param fields: highlight matching terms from any field
+    :param tag: optional html tag name
+    :param formatter: optional lucene Formatter
     :param encoder: optional lucene Encoder
-    :param field: optional field name used to match query terms
-    :param reader: optional lucene IndexReader to compute term weights
     """
-    def __init__(self, query, analyzer, span=True, formatter=None, encoder=None, field=None, reader=None):
-        if isinstance(formatter, basestring):
-            formatter = lucene.SimpleHTMLFormatter('<{0}>'.format(formatter), '</{0}>'.format(formatter))
-        scorer = (lucene.QueryScorer if span else lucene.QueryTermScorer)(*filter(None, [query, reader, field]))
+    def __init__(self, searcher, query, field, terms=False, fields=False, tag='', formatter=None, encoder=None):
+        if tag:
+            formatter = lucene.SimpleHTMLFormatter('<{0}>'.format(tag), '</{0}>'.format(tag))
+        scorer = (lucene.QueryTermScorer if terms else lucene.QueryScorer)(query, *(searcher.indexReader, field) * (not fields))
         lucene.Highlighter.__init__(self, *filter(None, [formatter, encoder, scorer]))
-        self.analyzer = analyzer
-    def fragments(self, text, count=1, field=None):
+        self.searcher, self.field = searcher, field
+        self.selector = lucene.MapFieldSelector([field])
+    def fragments(self, doc, count=1):
         """Return highlighted text fragments.
         
-        :param text: text string to be searched
+        :param doc: text string or doc id to be highlighted
         :param count: maximum number of fragments
-        :param field: optional field to use for text analysis
         """
-        return list(self.getBestFragments(self.analyzer, field, text, count))
+        if not isinstance(doc, basestring):
+            doc = self.searcher.doc(doc, self.selector)[self.field]
+        return doc and list(self.getBestFragments(self.searcher.analyzer, self.field, doc, count))
+
+class FastVectorHighlighter(getattr(lucene, 'FastVectorHighlighter', object)):
+    """Inherited lucene FastVectorHighlighter with stored query.
+    Fields must be stored and have term vectors with offsets and positions.
+    
+    :param searcher: `IndexSearcher`_ with stored term vectors
+    :param query: lucene Query
+    :param field: field name of text
+    :param terms: highlight any matching term in query regardless of position
+    :param fields: highlight matching terms from any field
+    :param tag: optional html tag name
+    :param fragListBuilder: optional lucene FragListBuilder
+    :param fragmentsBuilder: optional lucene FragmentsBuilder
+    """
+    def __init__(self, searcher, query, field, terms=False, fields=False, tag='', fragListBuilder=None, fragmentsBuilder=None):
+        if tag:
+            fragmentsBuilder = lucene.SimpleFragmentsBuilder(['<{0}>'.format(tag)], ['</{0}>'.format(tag)])
+        args = fragListBuilder or lucene.SimpleFragListBuilder(), fragmentsBuilder or lucene.SimpleFragmentsBuilder()
+        lucene.FastVectorHighlighter.__init__(self, not terms, not fields, *args)
+        self.searcher, self.field = searcher, field
+        self.query = self.getFieldQuery(query)
+    def fragments(self, id, count=1, size=100):
+        """Return highlighted text fragments.
+        
+        :param id: document id
+        :param count: maximum number of fragments
+        :param size: maximum number of characters in fragment
+        """
+        return list(self.getBestFragments(self.query, self.searcher.indexReader, id, self.field, size, count))
 
 class SpellChecker(dict):
     """Correct spellings and suggest words for queries.
@@ -299,7 +311,7 @@ class SpellChecker(dict):
 
 class SpellParser(lucene.PythonQueryParser):
     """Inherited lucene QueryParser which corrects spelling.
-    Assign a `searcher`_ attribute or override :meth:`correct` implementation.
+    Assign a searcher attribute or override :meth:`correct` implementation.
     """
     def correct(self, term):
         "Return term with text replaced as necessary."
