@@ -1,6 +1,38 @@
 """
 Restful json `CherryPy <http://cherrypy.org/>`_ server.
 
+The server script mounts a `WebSearcher`_ (read_only) or `WebIndexer`_ root.
+Standard `CherryPy configuration <http://www.cherrypy.org/wiki/ConfigAPI>`_ applies,
+and the provided `custom tools <#tools>`_ are also configurable.
+All request and response bodies are `application/json values <http://tools.ietf.org/html/rfc4627.html#section-2.1>`_.
+
+WebSearcher exposes resources for an IndexSearcher.
+In addition to search requests, it provides access to term and document information in the index.
+Note Lucene doc ids are ephemeral;  they should only be used across requests for the same index version.
+
+ * :meth:`/ <WebSearcher.index>`
+ * :meth:`/search <WebSearcher.search>`
+ * :meth:`/docs <WebSearcher.docs>`
+ * :meth:`/terms <WebSearcher.terms>`
+ * :meth:`/update <WebSearcher.update>`
+
+WebIndexer extends WebSearcher, exposing additional resources and methods for an Indexer.
+Single documents may be added, deleted, or replaced by a unique indexed field.
+Multiples documents may also be added or deleted by query at once.
+By default changes are not visible until the update resource is called to commit a new index version.
+If a near real-time Indexer is used (an experimental feature in Lucene), then changes are instantly searchable.
+In such cases a commit still hasn't occurred;  the index based :meth:`validation headers <validate>` shouldn't be used for caching.
+
+ * :meth:`/ <WebIndexer.index>`
+ * :meth:`/search <WebIndexer.search>`
+ * :meth:`/docs <WebIndexer.docs>`
+ * :meth:`/fields <WebIndexer.fields>`
+ * :meth:`/update <WebIndexer.update>`
+
+Custom servers should create and mount WebSearchers and WebIndexers as needed.
+:meth:`Caches <WebSearcher.update>` and :meth:`field settings <WebIndexer.fields>` can then be applied directly before `starting <#start>`_ the server.
+WebSearchers and WebIndexers can of course also be subclassed for custom interfaces.
+
 CherryPy and Lucene VM integration issues:
  * Monitors (such as autoreload) are not compatible with the VM unless threads are attached.
  * WorkerThreads must be also attached to the VM.
@@ -9,11 +41,10 @@ CherryPy and Lucene VM integration issues:
 """
 
 from future_builtins import filter, map
-import warnings
 import re
 import time
 import httplib
-import threading
+import heapq
 import collections
 import itertools, operator
 import os, optparse
@@ -37,21 +68,23 @@ def tool(hook):
 @tool('before_handler')
 def json_(indent=None, content_type='application/json', process_body=None):
     """Handle request bodies and responses in json format.
-
+    
     :param indent: indentation level for pretty printing
-    :param content_type: response content-type header
+    :param content_type: request media type and response content-type header
     :param process_body: optional function to process body into request.params
     """
     request = cherrypy.serving.request
-    headers = cherrypy.response.headers
-    if request.headers.get('content-type', '').endswith('json'):
-        with HTTPError(httplib.BAD_REQUEST, ValueError):
+    media_type = request.headers.get('content-type')
+    if media_type == content_type:
+        with HTTPError(httplib.BAD_REQUEST, ValueError, AttributeError):
             request.json = json.load(request.body)
         if process_body is not None:
             with HTTPError(httplib.BAD_REQUEST, TypeError):
                 request.params.update(process_body(request.json))
-    elif request.headers.get('content-type') == 'application/x-www-form-urlencoded':
-        headers['Warning'] = '199 lupyne "Content-Type application/x-www-form-urlencoded has been deprecated and replaced with application/json"'
+    elif media_type is not None:
+        message = "Received Content-Type header {0}; only {1} is supported.".format(media_type, content_type)
+        raise cherrypy.HTTPError(httplib.UNSUPPORTED_MEDIA_TYPE, message)
+    headers = cherrypy.response.headers
     handler = request.handler
     def json_handler(*args, **kwargs):
         body = handler(*args, **kwargs)
@@ -79,12 +112,12 @@ def time_():
 @tool('on_start_resource')
 def validate(methods=('GET', 'HEAD'), etag=True, last_modified=True, max_age=None, expires=None):
     """Return and validate caching headers for GET requests.
-
+    
     :param methods: only set headers for specified methods
-    :param etag: return weak entity tag based on index version and validate if-match headers
-    :param last_modified: return last-modified based on index timestamp and validate if-modified headers
-    :param max_age: return cache-control max-age and age header based on last update
-    :param expires: return expires header based on last update
+    :param etag: return weak entity tag header based on index version and validate if-match headers
+    :param last_modified: return last-modified header based on index timestamp and validate if-modified headers
+    :param max_age: return cache-control max-age and age headers based on last update timestamp
+    :param expires: return expires header offset from last update timestamp
     """
     request = cherrypy.serving.request
     headers = cherrypy.response.headers
@@ -101,6 +134,49 @@ def validate(methods=('GET', 'HEAD'), etag=True, last_modified=True, max_age=Non
         if expires is not None:
             headers['expires'] = formatdate(expires + request.app.root.updated, usegmt=True)
 
+@tool('before_handler')
+def params(**types):
+    "Convert specified request params."
+    params = cherrypy.request.params
+    with HTTPError(httplib.BAD_REQUEST, ValueError):
+        for key in set(types).intersection(params):
+            params[key] = types[key](params[key])
+
+def multi(value):
+    return value and value.split(',')
+
+class params:
+    "Parameter parsing."
+    @staticmethod
+    def q(searcher, q, **options):
+        options = dict((key.partition('.')[-1], options[key]) for key in options if key.startswith('q.'))
+        field = options.pop('field', [])
+        fields = [field] if isinstance(field, basestring) else field
+        fields = [name.partition('^')[::2] for name in fields]
+        if any(boost for name, boost in fields):
+            field = dict((name, float(boost or 1.0)) for name, boost in fields)
+        elif isinstance(field, basestring):
+            (field, boost), = fields
+        else:
+            field = [name for name, boost in fields] or ''
+        if 'type' in options:
+            with HTTPError(httplib.BAD_REQUEST, AttributeError):
+                return getattr(engine.Query, options['type'])(field, q)
+        for key in set(options) - set(['op', 'version']):
+            with HTTPError(httplib.BAD_REQUEST, ValueError):
+                options[key] = json.loads(options[key])
+        if q is not None:
+            with HTTPError(httplib.BAD_REQUEST, lucene.JavaError):
+                return searcher.parse(q, field=field, **options)
+    @staticmethod
+    def fields(searcher, fields=None, **options):
+        if fields is not None:
+            fields = dict.fromkeys(fields)
+        multi = options.get('fields.multi', ())
+        indexed = (field.split(':') for field in options.get('fields.indexed', ()))
+        indexed = dict((item[0], searcher.comparator(*item)) for item in indexed)
+        return fields, multi, indexed
+
 def json_error(version, **body):
     "Transform errors into json format."
     tool = cherrypy.request.toolmaps['tools'].get('json', {})
@@ -135,8 +211,8 @@ def HTTPError(status, *exceptions):
 
 class WebSearcher(object):
     "Dispatch root with a delegated Searcher."
-    _cp_config = dict.fromkeys(map('tools.{0}.on'.format, ['gzip', 'json', 'allow', 'time', 'validate']), True)
-    _cp_config.update({'error_page.default': json_error, 'tools.gzip.mime_types': ['text/html', 'text/plain', 'application/json']})
+    _cp_config = dict.fromkeys(map('tools.{0}.on'.format, ['gzip', 'accept', 'json', 'allow', 'time', 'validate']), True)
+    _cp_config.update({'error_page.default': json_error, 'tools.gzip.mime_types': ['text/html', 'text/plain', 'application/json'], 'tools.accept.media': 'application/json'})
     def __init__(self, *directories, **kwargs):
         self.searcher = engine.MultiSearcher(directories, **kwargs) if len(directories) > 1 else engine.IndexSearcher(*directories, **kwargs)
         self.updated = time.time()
@@ -152,45 +228,8 @@ class WebSearcher(object):
         self.__init__(*self.__dict__.pop('args'), **self.__dict__.pop('kwargs'))
     def close(self):
         self.searcher.close()
-    @staticmethod
-    def parse(searcher, q, **options):
-        "Return parsed query using q.* parser options."
-        options = dict((key.partition('.')[-1], options[key]) for key in options if key.startswith('q.'))
-        field = options.pop('field', [])
-        fields = [field] if isinstance(field, basestring) else field
-        fields = [name.partition('^')[::2] for name in fields]
-        if any(boost for name, boost in fields):
-            field = dict((name, float(boost or 1.0)) for name, boost in fields)
-        elif isinstance(field, basestring):
-            (field, boost), = fields
-        else:
-            field = [name for name, boost in fields] or ''
-        if 'type' in options:
-            with HTTPError(httplib.BAD_REQUEST, AttributeError):
-                return getattr(engine.Query, options['type'])(field, q)
-        for key in set(options) - set(['op', 'version']):
-            with HTTPError(httplib.BAD_REQUEST, ValueError):
-                options[key] = json.loads(options[key])
-        if q is not None:
-            with HTTPError(httplib.BAD_REQUEST, lucene.JavaError):
-                return searcher.parse(q, field=field, **options)
-    @staticmethod
-    def select(fields=None, **options):
-        "Return parsed field selectors: stored, multi-valued, and indexed."
-        if fields is not None:
-            fields = dict.fromkeys(filter(None, fields.split(',')))
-        if 'multifields' in options:
-            options['fields.multi'] = options.pop('multifields')
-            cherrypy.response.headers['Warning'] = '199 lupyne "multifields has been deprecated and renamed fields.multi"'
-        multi = list(filter(None, options.get('fields.multi', '').split(',')))
-        indexed = [field.split(':') for field in options.get('fields.indexed', '').split(',') if field]
-        return fields, multi, indexed
     @cherrypy.expose
-    @cherrypy.tools.allow(methods=['POST'])
-    def refresh(self, **caches):
-        raise cherrypy.HTTPRedirect(cherrypy.request.script_name + '/update', httplib.MOVED_PERMANENTLY)
-    @cherrypy.expose
-    @cherrypy.tools.json(process_body=dict.fromkeys)
+    @cherrypy.tools.json(process_body=lambda body: dict.fromkeys(body, True))
     @cherrypy.tools.allow(methods=['POST'])
     def update(self, **caches):
         """Refresh index version.
@@ -202,7 +241,7 @@ class WebSearcher(object):
             
             :return: *int*
         """
-        self.searcher = self.searcher.reopen(**dict.fromkeys(caches, True))
+        self.searcher = self.searcher.reopen(**caches)
         self.updated = time.time()
         return len(self.searcher)
     @cherrypy.expose
@@ -214,11 +253,11 @@ class WebSearcher(object):
             
             :return: {*string*: *int*,... }
         """
-        searcher = self.searcher
-        if isinstance(searcher, lucene.MultiSearcher):
-            return dict((unicode(reader.directory()), reader.numDocs()) for reader in searcher.sequentialSubReaders)
-        return {unicode(searcher.directory): len(searcher)}
+        reader = self.searcher.indexReader
+        readers = reader.sequentialSubReaders if lucene.MultiReader.instance_(reader) else [reader]
+        return dict((unicode(reader.directory()), reader.numDocs()) for reader in readers)
     @cherrypy.expose
+    @cherrypy.tools.params(fields=multi, **{'fields.multi': multi, 'fields.indexed': multi})
     def docs(self, *path, **options):
         """Return ids or documents.
         
@@ -238,20 +277,22 @@ class WebSearcher(object):
         searcher = self.searcher
         if not path:
             return list(searcher)
-        fields, multi, indexed = self.select(**options)
         with HTTPError(httplib.NOT_FOUND, ValueError):
             id, = map(int, path) if len(path) == 1 else searcher.docs(*path)
+        fields, multi, indexed = params.fields(searcher, **options)
         with HTTPError(httplib.NOT_FOUND, lucene.JavaError):
             doc = searcher[id] if fields is None else searcher.get(id, *itertools.chain(fields, multi))
         result = doc.dict(*multi, **(fields or {}))
-        result.update((item[0], searcher.comparator(*item)[id]) for item in indexed)
+        result.update((name, indexed[name][id]) for name in indexed)
         return result
     @cherrypy.expose
+    @cherrypy.tools.params(count=int, start=int, fields=multi, sort=multi, facets=multi, hl=multi, mlt=int, spellcheck=int, timeout=float,
+        **{'fields.multi': multi, 'fields.indexed': multi, 'facets.count': int, 'facets.min': int, 'group.count': int, 'group.limit': int, 'hl.count': int, 'mlt.fields': multi})
     def search(self, q=None, count=None, start=0, fields=None, sort=None, facets='', group='', hl='', mlt=None, spellcheck=0, timeout=None, **options):
         """Run query and return documents.
         
         **GET** /search?
-            Return list of document objects and total doc count.
+            Return array of document objects and total doc count.
             
             &q=\ *chars*\ &q.type=[term|prefix|wildcard]&q.\ *chars*\ =...,
                 query, optional type to skip parsing, and optional parser settings: q.field, q.op,...
@@ -270,8 +311,9 @@ class WebSearcher(object):
                 | field name, optional type, minus sign indicates descending
                 | optionally score docs, additionally compute maximum score
             
-            &facets=\ *chars*,...
-                include facet counts for given field names; facets filters are cached
+            &facets=\ *chars*,... &facets.count=\ *int*\&facets.min=0
+                | include facet counts for given field names; facets filters are cached
+                | optional maximum number of most populated facet values per field, and minimum count to return
             
             &group=\ *chars*\ [:*chars*]&group.count=1&group.limit=\ *int*
                 | group documents by field value with optional type, up to given maximum count
@@ -301,26 +343,14 @@ class WebSearcher(object):
                 | "maxscore": *number*\|null,
                 | "docs": [{"__id__": *int*, "__score__": *number*, "__highlights__": {*string*: *array*,... }, *string*: *string*\|\ *array*,... },... ],
                 | "facets": {*string*: {*string*: *int*,... },... },
-                | "groups": [{"count": *int*, "value": *value*, "docs": [{... },... ]},... ]
+                | "groups": [{"count": *int*, "value": *value*, "docs": [*object*,... ]},... ]
                 | "spellcheck": {*string*: {*string*: [*string*,... ],... },... },
                 | }
         """
         searcher = self.searcher
-        with HTTPError(httplib.BAD_REQUEST, ValueError):
-            start = int(start)
-            if count is not None:
-                count = int(count) + start
-            spellcheck = int(spellcheck)
-            if timeout is not None:
-                timeout = float(timeout)
-            gcount = int(options.get('group.count', 1))
-            glimit = int(options['group.limit']) if 'group.limit' in options else float('inf')
-            hlcount = int(options.get('hl.count', 1))
-            if mlt is not None:
-                mlt = int(mlt)
         reverse = False
         if sort is not None:
-            sort = (re.match('(-?)(\w+):?(\w*)', field).groups() for field in sort.split(','))
+            sort = (re.match('(-?)(\w+):?(\w*)', field).groups() for field in sort)
             sort = [(name, (type or 'string'), (reverse == '-')) for reverse, name, type in sort]
             if count is None:
                 with HTTPError(httplib.BAD_REQUEST, ValueError, AttributeError):
@@ -330,38 +360,39 @@ class WebSearcher(object):
             else:
                 with HTTPError(httplib.BAD_REQUEST, AttributeError):
                     sort = [searcher.sorter(name, type, reverse=reverse) for name, type, reverse in sort]
-        q = self.parse(searcher, q, **options)
+        q = params.q(searcher, q, **options)
         qfilter = options.pop('filter', None)
         if qfilter is not None and qfilter not in searcher.filters:
-            searcher.filters[qfilter] = engine.Query.__dict__['filter'](self.parse(searcher, qfilter, **options))
+            searcher.filters[qfilter] = engine.Query.__dict__['filter'](params.q(searcher, qfilter, **options))
         qfilter = searcher.filters.get(qfilter)
         if mlt is not None:
             if q is not None:
                 mlt = searcher.search(q, count=mlt+1, sort=sort, reverse=reverse).ids[mlt]
-            mltfields = filter(None, options.pop('mlt.fields', '').split(','))
+            mltfields = options.pop('mlt.fields', ())
             with HTTPError(httplib.BAD_REQUEST, ValueError):
                 attrs = dict((key.partition('.')[-1], json.loads(options[key])) for key in options if key.startswith('mlt.'))
             q = searcher.morelikethis(mlt, *mltfields, **attrs)
+        if count is not None:
+            count += start
         if count == 0:
             start = count = 1
         scores = options.get('sort.scores')
         scores = {'scores': scores is not None, 'maxscore': scores == 'max'}
         hits = searcher.search(q, filter=qfilter, count=count, sort=sort, reverse=reverse, timeout=timeout, **scores)[start:]
         result = {'query': q and unicode(q), 'count': hits.count, 'maxscore': hits.maxscore}
-        tag = options.get('hl.tag', 'strong')
-        field = 'fields' not in options.get('hl.enable', '') or None
-        span = 'terms' not in options.get('hl.enable', '')
+        tag, enable = options.get('hl.tag', 'strong'), options.get('hl.enable', '')
+        hlcount = options.get('hl.count', 1)
         if hl:
-            hl = dict((name, searcher.highlighter(q, span=span, field=(field and name), formatter=tag)) for name in hl.split(','))
-        fields, multi, indexed = self.select(fields, **options)
+            hl = dict((name, searcher.highlighter(q, name, terms='terms' in enable, fields='fields' in enable, tag=tag)) for name in hl)
+        fields, multi, indexed = params.fields(searcher, fields, **options)
         if fields is None:
             fields = {}
         else:
-            hits.fields = lucene.MapFieldSelector(list(itertools.chain(fields, multi, hl)))
-            fields = fields or {'__id__': None}
-        indexed = dict((item[0], searcher.comparator(*item)) for item in indexed)
+            hits.fields = lucene.MapFieldSelector(list(itertools.chain(fields, multi)))
         docs = []
         groups = collections.defaultdict(lambda: {'docs': [], 'count': 0, 'index': len(groups)})
+        gcount = options.get('group.count', 1)
+        glimit = options.get('group.limit', float('inf'))
         if group:
             with HTTPError(httplib.BAD_REQUEST, AttributeError):
                 group = searcher.comparator(*group.split(':'))
@@ -376,8 +407,9 @@ class WebSearcher(object):
         for hit in hits:
             doc = hit.dict(*multi, **fields)
             doc.update((name, indexed[name][hit.id]) for name in indexed)
+            fragments = (hl[name].fragments(hit.id, hlcount) for name in hl)
             if hl:
-                doc['__highlights__'] = dict((name, hl[name].fragments(hit[name], hlcount)) for name in hl if name in hit)
+                doc['__highlights__'] = dict((name, value) for name, value in zip(hl, fragments) if value is not None)
             (groups[group[hit.id]]['docs'] if group else docs).append(doc)
         for name in groups:
             groups[name]['value'] = name
@@ -387,14 +419,21 @@ class WebSearcher(object):
             result['docs'] = docs
         q = q or lucene.MatchAllDocsQuery()
         if facets:
-            facets = (tuple(facet.split(':')) if ':' in facet else facet for facet in facets.split(','))
-            result['facets'] = searcher.facets(q, *facets)
+            facets = (tuple(facet.split(':')) if ':' in facet else facet for facet in facets)
+            facets = result['facets'] = searcher.facets(q, *facets)
+            if 'facets.min' in options:
+                for name, counts in facets.items():
+                    facets[name] = dict((term, count) for term, count in counts.items() if count >= options['facets.min'])
+            if 'facets.count' in options:
+                for name, counts in facets.items():
+                    facets[name] = dict((term, counts[term]) for term in heapq.nlargest(options['facets.count'], counts, key=counts.__getitem__))
         if spellcheck:
             terms = result['spellcheck'] = collections.defaultdict(dict)
             for name, value in engine.Query.__dict__['terms'](q):
                 terms[name][value] = list(itertools.islice(searcher.correct(name, value), spellcheck))
         return result
     @cherrypy.expose
+    @cherrypy.tools.params(count=int, step=int)
     def terms(self, name='', value=':', *path, **options):
         """Return data about indexed terms.
         
@@ -422,12 +461,12 @@ class WebSearcher(object):
             :return: [*string*,... ]
         
         **GET** /terms/*chars*/*chars*
-            Return document count with given term.
+            Return document count for given term.
             
             :return: *int*
         
         **GET** /terms/*chars*/*chars*/docs
-            Return document ids with given term.
+            Return document ids for given term.
             
             :return: [*int*,... ]
         
@@ -448,19 +487,16 @@ class WebSearcher(object):
             with HTTPError(httplib.BAD_REQUEST, ValueError, AttributeError):
                 name, type = name.split(':')
                 type = getattr(__builtins__, type)
-                step = int(options.get('step', 0))
-            return list(searcher.numbers(name, step=step, type=type))
+            return list(searcher.numbers(name, step=options.get('step', 0), type=type))
         if ':' in value:
             with HTTPError(httplib.BAD_REQUEST, ValueError):
                 start, stop = value.split(':')
             return list(searcher.terms(name, start, stop or None))
         if 'count' in options:
-            with HTTPError(httplib.BAD_REQUEST, ValueError):
-                count = int(options['count'])
             if value.endswith('*'):
-                return searcher.suggest(name, value.rstrip('*'), count)
+                return searcher.suggest(name, value.rstrip('*'), options['count'])
             if value.endswith('~'):
-                return list(itertools.islice(searcher.correct(name, value.rstrip('~')), count))
+                return list(itertools.islice(searcher.correct(name, value.rstrip('~')), options['count']))
         if '*' in value or '?' in value:
             return list(searcher.terms(name, value))
         if '~' in value:
@@ -480,18 +516,22 @@ class WebSearcher(object):
         raise cherrypy.NotFound()
 
 class WebIndexer(WebSearcher):
-    "Dispatch root which extends searcher to include write methods."
-    commit = WebSearcher.refresh
+    "Dispatch root with a delegated Indexer, exposing write methods."
     def __init__(self, *args, **kwargs):
         self.indexer = engine.Indexer(*args, **kwargs)
         self.updated = time.time()
-        self.lock = threading.Lock()
     @property
     def searcher(self):
         return self.indexer.indexSearcher
     def close(self):
         self.indexer.close()
         WebSearcher.close(self)
+    def refresh(self):
+        if self.indexer.nrt:
+            self.indexer.refresh()
+            self.updated = time.time()
+        else:
+            cherrypy.response.status = httplib.ACCEPTED
     @cherrypy.expose
     @cherrypy.tools.json(process_body=lambda body: {'directories': list(body)})
     @cherrypy.tools.allow(methods=['GET', 'HEAD', 'POST'])
@@ -506,11 +546,9 @@ class WebIndexer(WebSearcher):
         if cherrypy.request.method == 'POST':
             for directory in directories:
                 self.indexer += directory
-            cherrypy.response.status = httplib.ACCEPTED
+            self.refresh()
         return {unicode(self.indexer.directory): len(self.indexer)}
     @cherrypy.expose
-    @cherrypy.tools.json(process_body=dict.fromkeys)
-    @cherrypy.tools.allow(methods=['POST'])
     def update(self, **options):
         """Commit index changes and refresh index version.
         
@@ -521,10 +559,10 @@ class WebIndexer(WebSearcher):
             
             :return: *int*
         """
-        with self.lock:
-            self.indexer.commit(**dict.fromkeys(options, True))
+        self.indexer.commit(**options)
         self.updated = time.time()
         return len(self.indexer)
+    update._cp_config = dict(WebSearcher.update._cp_config)
     @cherrypy.expose
     @cherrypy.tools.allow(methods=['GET', 'HEAD', 'POST', 'PUT', 'DELETE'])
     def docs(self, *path, **options):
@@ -551,19 +589,15 @@ class WebIndexer(WebSearcher):
             name, value = path
             doc = getattr(request, 'json', {})
             with HTTPError(httplib.CONFLICT, KeyError, AssertionError):
-                assert self.indexer.fields[name].index != lucene.Field.Index.NO, 'unique field must be indexed'
+                assert self.indexer.fields[name].index.indexed, 'unique field must be indexed'
+            with HTTPError(httplib.BAD_REQUEST, AssertionError):
                 assert doc.setdefault(name, value) == value, 'multiple values for unique field'
             self.indexer.update(name, value, doc)
         else:
-            docs = getattr(request, 'json', ())
-            with HTTPError(httplib.BAD_REQUEST, KeyError, ValueError): # deprecated
-                if isinstance(docs, dict):
-                    docs = docs['docs']
-                if 'docs' in options:
-                    docs = json.loads(options['docs'])
-            for doc in docs:
+            for doc in getattr(request, 'json', ()):
                 self.indexer.add(doc)
-        cherrypy.response.status = httplib.ACCEPTED
+        self.refresh()
+    docs._cp_config.update(WebSearcher.docs._cp_config)
     @cherrypy.expose
     @cherrypy.tools.allow(methods=['GET', 'HEAD', 'DELETE'])
     def search(self, q=None, **options):
@@ -577,8 +611,9 @@ class WebIndexer(WebSearcher):
         if q is None:
             self.indexer.deleteAll()
         else:
-            self.indexer.delete(self.parse(self.searcher, q, **options))
-        cherrypy.response.status = httplib.ACCEPTED
+            self.indexer.delete(params.q(self.searcher, q, **options))
+        self.refresh()
+    search._cp_config.update(WebSearcher.search._cp_config)
     @cherrypy.expose
     @cherrypy.tools.json(process_body=dict)
     @cherrypy.tools.allow(methods=['GET', 'HEAD', 'PUT'])
@@ -604,12 +639,13 @@ class WebIndexer(WebSearcher):
         if cherrypy.request.method == 'PUT':
             if name not in self.indexer.fields:
                 cherrypy.response.status = httplib.CREATED
-            self.indexer.set(name, **settings)
+            with HTTPError(httplib.BAD_REQUEST, AttributeError):
+                self.indexer.set(name, **settings)
         with HTTPError(httplib.NOT_FOUND, KeyError):
             field = self.indexer.fields[name]
         return dict((name, str(getattr(field, name))) for name in ['store', 'index', 'termvector'])
 
-def start(root=None, path='', config=None, pidfile='', daemonize=False, autoreload=0, autoupdate=0, callback=None, autorefresh=0):
+def start(root=None, path='', config=None, pidfile='', daemonize=False, autoreload=0, autoupdate=0, callback=None):
     """Attach root, subscribe to plugins, and start server.
     
     :param root,path,config: see cherrypy.quickstart
@@ -627,9 +663,6 @@ def start(root=None, path='', config=None, pidfile='', daemonize=False, autorelo
         cherrypy.process.plugins.Daemonizer(cherrypy.engine).subscribe()
     if autoreload:
         Autoreloader(cherrypy.engine, autoreload).subscribe()
-    if autorefresh:
-        warnings.warn('Autorefresh has been renamed autoupdate.', DeprecationWarning)
-        autoupdate = autorefresh
     if autoupdate:
         AttachedMonitor(cherrypy.engine, root.update, autoupdate).subscribe()
     if callback:
@@ -644,7 +677,6 @@ parser.add_option('-p', '--pidfile', metavar='FILE', help='store the process id
 parser.add_option('-d', '--daemonize', action='store_true', help='run the server as a daemon')
 parser.add_option('--autoreload', type=int, metavar='SECONDS', help='automatically reload modules; replacement for engine.autoreload')
 parser.add_option('--autoupdate', type=int, metavar='SECONDS', help='automatically update index version')
-parser.add_option('--autorefresh', type=int, metavar='SECONDS', help='deprecated; use autoupdate')
 
 if __name__ == '__main__':
     options, args = parser.parse_args()
