@@ -12,19 +12,12 @@ import lucene
 from lupyne import engine
 import fixture
 
-if issubclass(lucene.TokenFilter, collections.Iterable):
-    def typeAsPayload(tokens):
-        "Generator variant of lucene TypeAsPayloadTokenFilter."
-        for token in tokens:
-            token.payload = lucene.Payload(lucene.JArray_byte(token.type().encode('utf8')))
-            yield token
-else:
-    class typeAsPayload(engine.TokenFilter):
-        "Custom implementation of lucene TypeAsPayloadTokenFilter."
-        def incrementToken(self):
-            result = engine.TokenFilter.incrementToken(self)
-            self.payload = self.type.encode('utf8')
-            return result
+class typeAsPayload(engine.TokenFilter):
+    "Custom implementation of lucene TypeAsPayloadTokenFilter."
+    def incrementToken(self):
+        result = engine.TokenFilter.incrementToken(self)
+        self.payload = self.type.encode('utf8')
+        return result
 
 @contextmanager
 def assertWarns(*categories):
@@ -51,20 +44,19 @@ class TestCase(BaseTest):
     def testInterface(self):
         "Indexer and document interfaces."
         self.assertRaises(TypeError, engine.IndexSearcher)
-        analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT)
+        analyzer = lucene.StandardAnalyzer(lucene.Version.values()[-1])
         stemmer = engine.Analyzer(analyzer, lucene.PorterStemFilter, typeAsPayload)
         for token in stemmer.tokens('hello'):
             assert token.positionIncrement == 1
-            if not isinstance(token, lucene.Token):
-                assert engine.TokenFilter(lucene.EmptyTokenStream()).payload is None
-                assert token.term == 'hello'
-                assert token.type == token.payload == '<ALPHANUM>'
-                assert token.offset == (0, 5)
-                token.term = token.type = ''
-                token.offset, token.positionIncrement = (0, 0), 0
+            assert engine.TokenFilter(lucene.EmptyTokenStream()).payload is None
+            assert token.term == 'hello'
+            assert token.type == token.payload == '<ALPHANUM>'
+            assert token.offset == (0, 5)
+            token.term = token.type = ''
+            token.offset, token.positionIncrement = (0, 0), 0
         assert str(stemmer.parse('hellos', field=['body', 'title'])) == 'body:hello title:hello'
         assert str(stemmer.parse('hellos', field={'body': 1.0, 'title': 2.0})) == 'body:hello title:hello^2.0'
-        indexer = engine.Indexer(analyzer=stemmer)
+        indexer = engine.Indexer(analyzer=stemmer, version=lucene.Version.LUCENE_30)
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
         indexer.set('name', store=True, index=False, boost=2.0)
@@ -79,7 +71,8 @@ class TestCase(BaseTest):
         assert not indexer.optimized
         indexer.commit()
         assert searcher is not indexer.indexSearcher
-        searcher = engine.ParallelMultiSearcher([indexer.indexSearcher, indexer.directory])
+        with assertWarns(DeprecationWarning, DeprecationWarning):
+            searcher = engine.ParallelMultiSearcher([indexer.indexSearcher, indexer.directory])
         assert searcher.count() == 2 * len(indexer)
         assert list(indexer) == [0]
         assert indexer.current and indexer.optimized
@@ -131,6 +124,8 @@ class TestCase(BaseTest):
         assert str(query) == '+text:* +text:hello~0.5 text:hello~0.1'
         query = engine.Query.span('text', 'world')
         assert str(query.mask('name')) == 'mask(text:world) as name'
+        if hasattr(lucene, 'SpanPositionCheckQuery'):
+            assert str(query.payload()) == 'spanPayCheck(text:world, payloadRef: )'
         query = engine.Query.disjunct(0.1, query, name='sample')
         assert str(query) == '(text:world | name:sample)~0.1'
         query = engine.Query.near('text', 'hello', ('tag', 'python'), slop=-1, inOrder=False)
@@ -139,18 +134,20 @@ class TestCase(BaseTest):
         (doc, items), = indexer.spans(query, payloads=True)
         (start, stop, payloads), = items
         assert doc == 0 and start == 0 and stop == 2 and payloads == ['<ALPHANUM>', '<ALPHANUM>']
+        if hasattr(lucene, 'SpanPositionCheckQuery'):
+            (doc, count), = indexer.spans(query.payload('<ALPHANUM>', '<ALPHANUM>'))
+            assert doc == 0 and count == 1
+            assert not indexer.search(query.payload('<>'))
         query = engine.Query.near('text', 'hello', 'world', collectPayloads=False)
         (doc, items), = indexer.spans(query, payloads=True)
         assert doc == 0 and items == []
         indexer.delete('name:sample')
         indexer.delete('tag', 'python')
-        assert 0 in indexer
-        assert len(indexer) == sum(indexer.segments.values())
+        assert 0 in indexer and len(indexer) == 1 and indexer.segments == {'_0': 1}
         indexer.commit()
-        assert len(indexer) < sum(indexer.segments.values())
-        indexer.optimize()
-        assert len(indexer) == sum(indexer.segments.values())
-        assert 0 not in indexer
+        assert 0 not in indexer and len(indexer) == 0 and sum(indexer.segments.values()) == 0
+        with assertWarns(DeprecationWarning):
+            assert isinstance(super(engine.Indexer, indexer).segments, dict)
         indexer.add(tag='test', name='old')
         indexer.update('tag', boost=2.0, tag='test')
         indexer.commit()
@@ -177,7 +174,6 @@ class TestCase(BaseTest):
         indexer.commit(expunge=True)
         assert not indexer.hasDeletions()
         indexer.commit(optimize=2)
-        len(indexer.segments) <= 2
         indexer.commit(optimize=True)
         assert indexer.optimized
         del reader.indexReader
@@ -198,12 +194,7 @@ class TestCase(BaseTest):
         engine.IndexSearcher.load(searcher.directory) # ensure directory isn't closed
         assert len(indexer) == len(searcher) and lucene.RAMDirectory.instance_(searcher.directory)
         assert indexer.filters == indexer.spellcheckers == {}
-        with assertWarns(DeprecationWarning):
-            assert indexer.facets([0], 'amendment')
-            assert not indexer.facets(lucene.OpenBitSet())
-            assert not indexer.search(filter=[])
-            f = engine.Filter([])
-            assert not engine.Filter.overlap(f, f)
+        assert indexer.facets(lucene.MatchAllDocsQuery(), 'amendment')
         assert indexer.suggest('amendment', '')
         assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
         indexer.delete('amendment', doc['amendment'])
@@ -211,8 +202,9 @@ class TestCase(BaseTest):
         indexer.commit(filters=True, spellcheckers=True)
         assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
         doc['amendment'] = engine.Analyzer(lucene.WhitespaceTokenizer).tokens(doc['amendment'])
-        scores = list(searcher.match(doc, 'text:congress', 'text:law', 'amendment:27'))
-        assert 0.0 == scores[0] < scores[1] < scores[2] < 1.0
+        doc['date'] = engine.Analyzer(lucene.WhitespaceTokenizer).tokens(doc['date']), 2.0
+        scores = list(searcher.match(doc, 'text:congress', 'text:law', 'amendment:27', 'date:19*'))
+        assert 0.0 == scores[0] < scores[1] < scores[2] < scores[3] == 1.0
         searcher = engine.MultiSearcher([indexer.directory, self.tempdir])
         assert searcher.count() == len(searcher) == 2 * len(indexer)
         searcher = searcher.reopen()
@@ -250,6 +242,8 @@ class TestCase(BaseTest):
         assert len(hits) == 5 and hits.count == 8
         assert not any(map(math.isnan, hits.scores))
         assert hits.maxscore == max(hits.scores)
+        hits = indexer.search('text:people', count=5, sort=lucene.Sort.INDEXORDER)
+        assert sorted(hits.ids) == hits.ids
         sort = engine.SortField('amendment', type=int)
         hits = indexer.search('text:people', count=5, sort=sort)
         assert [hit.get('amendment') for hit in hits] == [None, None, '1', '2', '4']
@@ -314,6 +308,8 @@ class TestCase(BaseTest):
         near = span.near(engine.Query.span('text', 'papers') | engine.Query.span('text', 'things'), slop=1)
         assert indexer.count(span - near) == count - 1
         assert 0 < indexer.count(span[:100]) < count
+        if hasattr(lucene, 'SpanPositionRangeQuery'):
+            assert 0 < indexer.count(span[50:100]) == indexer.count(span[:100] - span[:50]) < indexer.count(span[:100])
         spans = dict(indexer.spans(span))
         assert len(spans) == count and spans == dict(indexer.docs('text', 'persons', counts=True))
         near = engine.Query.near('text', 'persons', 'papers', slop=2)
@@ -358,9 +354,6 @@ class TestCase(BaseTest):
         indexer = engine.Indexer(self.tempdir)
         for name, params in fixture.zipcodes.fields.items():
             indexer.set(name, **params)
-        with assertWarns(DeprecationWarning, DeprecationWarning):
-            indexer.set('longitude', engine.PrefixField, store=True)
-            assert engine.NestedField('state:county:city').sep == ':'
         indexer.fields['location'] = engine.NestedField('state.county.city')
         for doc in fixture.zipcodes.docs():
             if doc['state'] in ('CA', 'AK', 'WY', 'PR'):
@@ -370,13 +363,6 @@ class TestCase(BaseTest):
         indexer.commit()
         assert set(['state', 'zipcode']) < set(indexer.names('indexed'))
         assert set(['latitude', 'longitude', 'county', 'city']) == set(indexer.names('unindexed'))
-        longitude = max(name for name in indexer.names() if name.startswith('longitude'))
-        lngs = list(indexer.terms(longitude))
-        east, west = lngs[0], lngs[-1]
-        hit, = indexer.search(engine.Query.term(longitude, west))
-        assert hit['state'] == 'AK' and hit['county'] == 'Aleutians West'
-        hit, = indexer.search(engine.Query.term(longitude, east))
-        assert hit['state'] == 'PR' and hit['county'] == 'Culebra'
         states = list(indexer.terms('state'))
         assert states[0] == 'AK' and states[-1] == 'WY'
         counties = [term.split('.')[-1] for term in indexer.terms('state.county', 'CA', 'CA~')]
@@ -389,19 +375,9 @@ class TestCase(BaseTest):
         assert sorted(set(hit['city'] for hit in hits)) == cities
         assert cities[0] == 'Acton' and cities[-1] == 'Woodland Hills'
         hit, = indexer.search('zipcode:90210')
-        assert hit['state'] == 'CA' and hit['county'] == 'Los Angeles' and hit['city'] == 'Beverly Hills'
-        assert hit['longitude'] == '-118.406'
-        lng = hit['longitude'][:4]
-        field = indexer.fields['longitude']
-        hits = indexer.search(field.prefix(lng))
-        assert hit.id in hits.ids
-        assert len(hits) == indexer.count(engine.Query.prefix(longitude, lng))
-        count = indexer.count(field.prefix(lng[:3]))
-        assert count > len(hits)
-        assert count == indexer.count(field.range(lng[:3], lng[:3]+'~'))
-        assert count > indexer.count(engine.Query.term('state', 'CA'), filter=engine.Query.term(longitude, lng).filter())
+        assert hit['state'] == 'CA' and hit['county'] == 'Los Angeles' and hit['city'] == 'Beverly Hills' and hit['longitude'] == '-118.406'
         query = engine.Query.prefix('zipcode', '90')
-        (field, facets), = indexer.facets(query, 'state.county').items()
+        (field, facets), = indexer.facets(query.filter(), 'state.county').items()
         assert field == 'state.county'
         la, orange = sorted(filter(facets.get, facets))
         assert la == 'CA.Los Angeles' and facets[la] > 100
@@ -468,6 +444,13 @@ class TestCase(BaseTest):
     
     def testFields(self):
         "Custom fields."
+        self.assertRaises(lucene.JavaError, engine.Field, '', store='invalid')
+        self.assertRaises(AttributeError, engine.Field, '', omit='value')
+        self.assertRaises(lucene.JavaError, engine.Field, '', index=False)
+        field = engine.Field('', index=True, analyzed=True, omitNorms=True, termvector=True, withPositions=True, withOffsets=True)
+        field, = field.items(' ')
+        attrs = 'indexed', 'tokenized', 'termVectorStored', 'storePositionWithTermVector', 'storeOffsetWithTermVector', 'omitNorms'
+        assert all(getattr(field, attr) for attr in attrs)
         indexer = engine.Indexer(self.tempdir)
         indexer.set('amendment', engine.FormatField, format='{0:02d}', store=True)
         indexer.set('size', engine.FormatField, format='{0:04d}', store=True)
@@ -517,10 +500,9 @@ class TestCase(BaseTest):
     def testNumeric(self):
         "Numeric fields."
         indexer = engine.Indexer(self.tempdir)
-        with assertWarns(DeprecationWarning):
-            indexer.set('amendment', engine.numeric.NumericField, store=True)
-            indexer.set('date', engine.numeric.DateTimeField, store=True)
-            indexer.set('size', engine.numeric.NumericField, store=True, step=5)
+        indexer.set('amendment', engine.NumericField, store=True)
+        indexer.set('date', engine.DateTimeField, store=True)
+        indexer.set('size', engine.NumericField, store=True, step=5)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
                 indexer.add(amendment=int(doc['amendment']), date=[tuple(map(int, doc['date'].split('-')))], size=len(doc['text']))
@@ -567,25 +549,49 @@ class TestCase(BaseTest):
     def testHighlighting(self):
         "Highlighting text fragments."
         indexer = engine.Indexer()
-        indexer.set('text', store=True)
-        amendments = dict((doc['amendment'], doc['text']) for doc in fixture.constitution.docs() if 'amendment' in doc)
-        for amendment in amendments.values():
-            fragments = indexer.highlight('persons', amendment)
-            assert len(fragments) == ('persons' in amendment)
-            for fragment in fragments:
-                assert '<B>persons</B>' in fragment
-        text = amendments['4']
-        query = '"persons, houses, papers"'
-        fragments = indexer.highlight(query, text, count=3, span=False, formatter=lucene.SimpleHTMLFormatter('*', '*'))
-        assert len(fragments) == 2 and fragments[0].count('*') == 2*3 and '*persons*' in fragments[1]
-        fragment, = indexer.highlight(query, text, count=3, span=False, textFragmenter=lucene.SimpleFragmenter(200))
-        assert len(fragment) > len(text) and fragment.count('<B>persons</B>') == 2
-        fragment, = indexer.highlight(query, text, count=3, formatter='em')
-        assert len(fragment) < len(text) and fragment.count('<em>') == 3
-        indexer.add(text=text)
+        indexer.set('text', store=True, termvector=True, withPositions=True, withOffsets=True)
+        for doc in fixture.constitution.docs():
+            if 'amendment' in doc:
+                indexer.add(text=doc['text'])
         indexer.commit()
-        fragment, = indexer.highlight(query, 0, field='text')
-        assert fragment.count('<B>') == fragment.count('</B>') == 3
+        highlighter = indexer.highlighter('persons', 'text')
+        for id in indexer:
+            fragments = highlighter.fragments(id)
+            assert len(fragments) == ('persons' in indexer[id]['text'])
+            assert all('<b>persons</b>' in fragment.lower() for fragment in fragments)
+        id = 3
+        text = indexer[id]['text']
+        query = '"persons, houses, papers"'
+        highlighter = indexer.highlighter(query, '', terms=True, fields=True, formatter=lucene.SimpleHTMLFormatter('*', '*'))
+        fragments = highlighter.fragments(text, count=3)
+        assert len(fragments) == 2 and fragments[0].count('*') == 2*3 and '*persons*' in fragments[1]
+        highlighter = indexer.highlighter(query, '', terms=True)
+        highlighter.textFragmenter = lucene.SimpleFragmenter(200)
+        fragment, = highlighter.fragments(text, count=3)
+        assert len(fragment) > len(text) and fragment.count('<B>persons</B>') == 2
+        fragment, = indexer.highlighter(query, 'text', tag='em').fragments(id, count=3)
+        assert len(fragment) < len(text) and fragment.index('<em>persons') < fragment.index('papers</em>')
+        fragment, = indexer.highlighter(query, 'text').fragments(id)
+        if hasattr(lucene, 'FastVectorHighlighter'):
+            assert fragment.count('<b>') == fragment.count('</b>') == 1
+            highlighter = indexer.highlighter(query, 'text', fragListBuilder=lucene.SingleFragListBuilder())
+            text, = highlighter.fragments(id)
+            assert fragment in text and len(text) > len(fragment)
+        else:
+            assert fragment.count('<B>') == fragment.count('</B>') == 3
+    
+    def testNearRealTime(self):
+        "Near real-time index updates."
+        indexer = engine.Indexer(version=lucene.Version.LUCENE_30, nrt=True)
+        indexer.add()
+        assert indexer.count() == 0 and not indexer.current
+        indexer.refresh(filters=True)
+        assert indexer.count() == 1 and indexer.current
+        searcher = engine.IndexSearcher(indexer.directory)
+        assert searcher.count() == 0 and searcher.current
+        indexer.add()
+        indexer.commit()
+        assert indexer.count() == engine.IndexSearcher(indexer.directory).count() == 2
 
 if __name__ == '__main__':
     lucene.initVM()
