@@ -1,6 +1,5 @@
 from future_builtins import map
 import unittest
-import warnings
 import os, sys
 import signal, subprocess
 import operator
@@ -9,14 +8,14 @@ import math
 import json
 import time, calendar
 import socket, errno
-from contextlib import contextmanager
+import contextlib
 from email.utils import parsedate
 import lucene
 import cherrypy
 from lupyne import client, engine, server
 import fixture, local
 
-@contextmanager
+@contextlib.contextmanager
 def assertRaises(exception, code):
     "Assert an exception is raised with specific code."
     try:
@@ -26,6 +25,16 @@ def assertRaises(exception, code):
     else:
         raise AssertionError(exception.__name__ + ' not raised')
 
+class Resource(client.Resource):
+    "Modify status and inject headers to test warning framework."
+    def getresponse(self):
+        response = client.Resource.getresponse(self)
+        response.msg.addheader('warning', '199 lupyne "test warning"')
+        if response.status == httplib.NOT_FOUND:
+            response.status = httplib.MOVED_PERMANENTLY
+            response.msg.addheader('location', 'http://{0}:{1}'.format(self.host, self.port))
+        return response
+
 class BaseTest(local.BaseTest):
     ports = 8080, 8081
     config = {'server.socket_timeout': 2, 'server.shutdown_timeout': 1}
@@ -86,6 +95,12 @@ class TestCase(BaseTest):
             resource.put('/docs/0')
         with assertRaises(httplib.HTTPException, httplib.METHOD_NOT_ALLOWED):
             resource.delete('/docs')
+        httplib.HTTPConnection.request(resource, 'POST', '/docs', headers={'content-length': '0', 'content-type': 'application/json'})
+        assert resource.getresponse().status == httplib.BAD_REQUEST
+        httplib.HTTPConnection.request(resource, 'POST', '/docs', headers={'content-length': '0', 'content-type': 'application/x-www-form-urlencoded'})
+        assert resource.getresponse().status == httplib.UNSUPPORTED_MEDIA_TYPE
+        httplib.HTTPConnection.request(resource, 'GET', '/', headers={'accept': 'text/html'})
+        assert resource.getresponse().status == httplib.NOT_ACCEPTABLE
         assert resource.get('/docs') == []
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
             resource.get('/docs/0')
@@ -145,11 +160,6 @@ class TestCase(BaseTest):
         assert resource.get('/docs') == [0]
         assert resource.get('/docs/0') == resource.get('/docs/name/sample') == {'name': 'sample'}
         assert resource.get('/docs/0', fields='missing') == {'missing': None}
-        with local.assertWarns(UserWarning, DeprecationWarning, UserWarning):
-            assert resource.get('/docs/0', multifields='name') == {'name': ['sample']}
-            assert not resource.post('/docs', docs=[])
-            httplib.HTTPConnection.request(resource, 'POST', '/docs', 'docs=[]', {'content-length': '7', 'content-type': 'application/x-www-form-urlencoded'})
-            assert not resource.getresponse()()
         assert resource.get('/docs/0', fields='', **{'fields.multi': 'missing'}) == {'missing': []}
         assert resource.get('/terms') == ['name', 'text']
         assert resource.get('/terms', option='unindexed') == []
@@ -208,10 +218,6 @@ class TestCase(BaseTest):
             resource.get('/docs/name/sample')
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
             resource.get('/fields')
-        with assertRaises(httplib.HTTPException, httplib.MOVED_PERMANENTLY):
-            resource.post('/refresh', ['spellcheckers'])
-        with local.assertWarns(DeprecationWarning):
-            assert resource.call('POST', '/refresh', redirect=True)
         resource = client.Resource('localhost', self.ports[0])
         assert not resource.delete('/search', q='sample', **{'q.field': 'name', 'q.type': 'term'})
         assert resource.get('/docs') == [0]
@@ -223,23 +229,25 @@ class TestCase(BaseTest):
         assert not resource.put('/docs/name/sample')
         assert resource.post('/update')
         assert resource.get('/docs/name/sample')
+        with assertRaises(httplib.HTTPException, httplib.BAD_REQUEST):
+            assert resource.put('/fields/name', {'omit': True})
+        with assertRaises(httplib.HTTPException, httplib.BAD_REQUEST):
+            resource.put('/docs/name/sample', {'name': 'mismatched'})
         with assertRaises(httplib.HTTPException, httplib.CONFLICT):
             resource.put('/docs/missing/sample')
+        assert resource.put('/fields/name', {'store': True, 'index': False, 'omitNorms': True})
         with assertRaises(httplib.HTTPException, httplib.CONFLICT):
-            resource.put('/docs/missing/sample', {'name': 'mismatched'})
-        assert resource.put('/fields/name', {'index': False})
-        with assertRaises(httplib.HTTPException, httplib.CONFLICT):
-            resource.put('/docs/missing/sample')
+            resource.put('/docs/name/sample')
         assert not resource.delete('/docs/missing/sample')
         resource.post('/update')
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
             resource.get('/docs/missing/sample')
         assert not resource.delete('/search')
-        with assertRaises(httplib.HTTPException, httplib.MOVED_PERMANENTLY):
-            resource.post('/commit', ['spellcheckers'])
         responses = resource.multicall(('POST', '/docs', [{}]), ('POST', '/update'), ('GET', '/docs'))
         assert responses[0].status == httplib.ACCEPTED and responses[1]() == len(responses[2]()) == 1
         assert resource.post('/', [self.tempdir]).values() == [2]
+        with local.assertWarns(DeprecationWarning, UserWarning):
+            assert Resource(resource.host, resource.port).call('GET', '/missing', redirect=True)()
         resource = client.Resource('localhost', self.ports[-1] + 1)
         with assertRaises(socket.error, errno.ECONNREFUSED):
             resource.get('/')
@@ -251,8 +259,6 @@ class TestCase(BaseTest):
     
     def testBasic(self):
         "Remote text indexing and searching."
-        with local.assertWarns(DeprecationWarning):
-            self.assertRaises(AttributeError, server.start, autorefresh=1)
         self.servers.append(self.start(self.ports[0], self.tempdir))
         resource = client.Resource('localhost', self.ports[0])
         assert resource.get('/fields') == []
@@ -410,9 +416,41 @@ class TestCase(BaseTest):
         result = resource.get('/search', count=0, facets='county.city:Los Angeles.*')
         facets = result['facets']['county.city']
         assert result['count'] > sum(facets.values()) and all(location.startswith('Los Angeles.') for location in facets)
+        result = resource.get('/search', count=0, facets='county', **{'facets.count': 3})
+        assert sorted(result['facets']['county']) == ['Los Angeles', 'Orange', 'San Diego']
+        result = resource.get('/search', count=0, facets='county', **{'facets.min': 140})
+        assert sorted(result['facets']['county']) == ['Los Angeles', 'Orange', 'San Diego']
         result = resource.get('/search', q='Los Angeles', group='county.city', **{'group.count': 0, 'q.field': 'county', 'q.type': 'prefix'})
         assert all(group['value'].startswith('Los Angeles') for group in result['groups'])
         assert sum(map(operator.itemgetter('count'), result['groups'])) == sum(facets.values()) == result['count']
+    
+    def testNearRealTime(self):
+        "Near Real Time updating and searching."
+        with contextlib.closing(engine.IndexWriter(self.tempdir)) as writer:
+            writer.add()
+        port = self.ports[0]
+        cherrypy.engine.subscribe('start_thread', server.attach_thread)
+        cherrypy.config.update(self.config)
+        cherrypy.config.update({'engine.autoreload.on': False, 'server.socket_port': port, 'tools.validate.expires': 0, 'tools.validate.last_modified': False})
+        root = server.WebIndexer(nrt=True)
+        cherrypy.tree.mount(root, config={'global': {}})
+        cherrypy.engine.start()
+        resource = client.Resource('localhost', port)
+        response = resource.call('GET', '/docs')
+        version, modified, expires = map(response.getheader, ('etag', 'last-modified', 'expires'))
+        assert modified is None and response() == []
+        assert resource.call('POST', '/docs', [{}]).status == httplib.OK
+        assert resource.get('/docs') == [0]
+        assert resource.call('DELETE', '/search').status == httplib.OK
+        assert resource.get('/docs') == []
+        time.sleep(max(0, calendar.timegm(parsedate(expires)) + 1 - time.time()))
+        assert resource.call('POST', '/', [self.tempdir]).status == httplib.OK
+        response = resource.call('GET', '/docs')
+        assert response() == [0] and expires != response.getheader('expires')
+        resource.post('/update')
+        response = resource.call('GET', '/docs')
+        assert response and version != response.getheader('etag')
+        cherrypy.engine.exit()
 
 if __name__ == '__main__':
     lucene.initVM()
